# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'login_dialog_ui.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_UserLoginDialog(object):
    def setupUi(self, UserLoginDialog):
        UserLoginDialog.setObjectName("UserLoginDialog")
        UserLoginDialog.resize(332, 232)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(UserLoginDialog.sizePolicy().hasHeightForWidth())
        UserLoginDialog.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(UserLoginDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame_2 = QtWidgets.QFrame(UserLoginDialog)
        self.frame_2.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.frame_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.frame_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_2.setFont(font)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.LoginLE = QtWidgets.QLineEdit(self.frame_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.LoginLE.sizePolicy().hasHeightForWidth())
        self.LoginLE.setSizePolicy(sizePolicy)
        self.LoginLE.setMinimumSize(QtCore.QSize(240, 44))
        self.LoginLE.setMaximumSize(QtCore.QSize(240, 16777215))
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setBold(True)
        font.setWeight(75)
        self.LoginLE.setFont(font)
        self.LoginLE.setText("")
        self.LoginLE.setAlignment(QtCore.Qt.AlignCenter)
        self.LoginLE.setClearButtonEnabled(False)
        self.LoginLE.setObjectName("LoginLE")
        self.horizontalLayout_2.addWidget(self.LoginLE)
        self.verticalLayout.addWidget(self.frame_2)
        self.frame = QtWidgets.QFrame(UserLoginDialog)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.frame)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_3 = QtWidgets.QLabel(self.frame)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_3.setFont(font)
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout.addWidget(self.label_3)
        self.PassLE = QtWidgets.QLineEdit(self.frame)
        self.PassLE.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.PassLE.sizePolicy().hasHeightForWidth())
        self.PassLE.setSizePolicy(sizePolicy)
        self.PassLE.setMinimumSize(QtCore.QSize(240, 44))
        self.PassLE.setMaximumSize(QtCore.QSize(240, 16777215))
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setBold(True)
        font.setWeight(75)
        self.PassLE.setFont(font)
        self.PassLE.setAlignment(QtCore.Qt.AlignCenter)
        self.PassLE.setObjectName("PassLE")
        self.horizontalLayout.addWidget(self.PassLE)
        self.verticalLayout.addWidget(self.frame)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.EnterPB = QtWidgets.QPushButton(UserLoginDialog)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.EnterPB.setFont(font)
        self.EnterPB.setObjectName("EnterPB")
        self.verticalLayout.addWidget(self.EnterPB)
        self.ExitPB = QtWidgets.QPushButton(UserLoginDialog)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.ExitPB.setFont(font)
        self.ExitPB.setObjectName("ExitPB")
        self.verticalLayout.addWidget(self.ExitPB)

        self.retranslateUi(UserLoginDialog)
        QtCore.QMetaObject.connectSlotsByName(UserLoginDialog)

    def retranslateUi(self, UserLoginDialog):
        _translate = QtCore.QCoreApplication.translate
        UserLoginDialog.setWindowTitle(_translate("UserLoginDialog", "GB Чат"))
        self.label_2.setText(_translate("UserLoginDialog", "Логин:"))
        self.LoginLE.setPlaceholderText(_translate("UserLoginDialog", "Логин"))
        self.label_3.setText(_translate("UserLoginDialog", "Пароль:"))
        self.PassLE.setPlaceholderText(_translate("UserLoginDialog", "Пароль"))
        self.EnterPB.setText(_translate("UserLoginDialog", "ВОЙТИ"))
        self.ExitPB.setText(_translate("UserLoginDialog", "УЙТИ"))
