# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main_window_ui.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        MainWindow.setMinimumSize(QtCore.QSize(800, 600))
        MainWindow.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.ContactFrame = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ContactFrame.sizePolicy().hasHeightForWidth())
        self.ContactFrame.setSizePolicy(sizePolicy)
        self.ContactFrame.setMinimumSize(QtCore.QSize(200, 0))
        self.ContactFrame.setMaximumSize(QtCore.QSize(300, 16777215))
        self.ContactFrame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.ContactFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.ContactFrame.setObjectName("ContactFrame")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.ContactFrame)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.usersTab = QtWidgets.QTabWidget(self.ContactFrame)
        self.usersTab.setEnabled(True)
        self.usersTab.setMaximumSize(QtCore.QSize(300, 16777215))
        font = QtGui.QFont()
        font.setPointSize(14)
        self.usersTab.setFont(font)
        self.usersTab.setElideMode(QtCore.Qt.ElideRight)
        self.usersTab.setObjectName("usersTab")
        self.Conts = QtWidgets.QWidget()
        self.Conts.setObjectName("Conts")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.Conts)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.ContactList = QtWidgets.QListView(self.Conts)
        self.ContactList.setObjectName("ContactList")
        self.verticalLayout_4.addWidget(self.ContactList)
        self.UpdateContsBT = QtWidgets.QPushButton(self.Conts)
        self.UpdateContsBT.setObjectName("UpdateContsBT")
        self.verticalLayout_4.addWidget(self.UpdateContsBT)
        self.usersTab.addTab(self.Conts, "")
        self.Users = QtWidgets.QWidget()
        self.Users.setObjectName("Users")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.Users)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.UserList = QtWidgets.QListView(self.Users)
        self.UserList.setObjectName("UserList")
        self.verticalLayout_2.addWidget(self.UserList)
        self.UpdateUsersBT = QtWidgets.QPushButton(self.Users)
        self.UpdateUsersBT.setObjectName("UpdateUsersBT")
        self.verticalLayout_2.addWidget(self.UpdateUsersBT)
        self.usersTab.addTab(self.Users, "")
        self.verticalLayout_3.addWidget(self.usersTab)
        self.horizontalLayout_2.addWidget(self.ContactFrame)
        self.MsgFrame = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.MsgFrame.sizePolicy().hasHeightForWidth())
        self.MsgFrame.setSizePolicy(sizePolicy)
        self.MsgFrame.setMinimumSize(QtCore.QSize(200, 400))
        self.MsgFrame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.MsgFrame.setFrameShadow(QtWidgets.QFrame.Plain)
        self.MsgFrame.setObjectName("MsgFrame")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.MsgFrame)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame_2 = QtWidgets.QFrame(self.MsgFrame)
        self.frame_2.setMinimumSize(QtCore.QSize(0, 30))
        self.frame_2.setMaximumSize(QtCore.QSize(16777215, 30))
        font = QtGui.QFont()
        font.setPointSize(14)
        self.frame_2.setFont(font)
        self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.frame_2)
        self.horizontalLayout.setContentsMargins(4, 0, 4, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.ContactStatus = QtWidgets.QLabel(self.frame_2)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(False)
        font.setWeight(50)
        self.ContactStatus.setFont(font)
        self.ContactStatus.setObjectName("ContactStatus")
        self.horizontalLayout.addWidget(self.ContactStatus)
        spacerItem = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.ContactName = QtWidgets.QLabel(self.frame_2)
        self.ContactName.setObjectName("ContactName")
        self.horizontalLayout.addWidget(self.ContactName)
        spacerItem1 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.SettingsBT = QtWidgets.QPushButton(self.frame_2)
        self.SettingsBT.setObjectName("SettingsBT")
        self.horizontalLayout.addWidget(self.SettingsBT)
        self.verticalLayout.addWidget(self.frame_2)
        self.frame = QtWidgets.QFrame(self.MsgFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setSpacing(0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.MsgField = QtWidgets.QListView(self.frame)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.MsgField.setFont(font)
        self.MsgField.setObjectName("MsgField")
        self.verticalLayout_5.addWidget(self.MsgField)
        self.frame_3 = QtWidgets.QFrame(self.frame)
        self.frame_3.setMinimumSize(QtCore.QSize(0, 40))
        self.frame_3.setMaximumSize(QtCore.QSize(16777215, 70))
        self.frame_3.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_3.setObjectName("frame_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.frame_3)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.EnterTE = QtWidgets.QTextEdit(self.frame_3)
        font = QtGui.QFont()
        font.setFamily("Segoe UI")
        font.setPointSize(12)
        self.EnterTE.setFont(font)
        self.EnterTE.setObjectName("EnterTE")
        self.horizontalLayout_3.addWidget(self.EnterTE)
        self.EnterBT = QtWidgets.QPushButton(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.EnterBT.sizePolicy().hasHeightForWidth())
        self.EnterBT.setSizePolicy(sizePolicy)
        self.EnterBT.setMinimumSize(QtCore.QSize(40, 40))
        font = QtGui.QFont()
        font.setPointSize(14)
        self.EnterBT.setFont(font)
        self.EnterBT.setObjectName("EnterBT")
        self.horizontalLayout_3.addWidget(self.EnterBT)
        self.verticalLayout_5.addWidget(self.frame_3)
        self.verticalLayout.addWidget(self.frame)
        self.horizontalLayout_2.addWidget(self.MsgFrame)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        self.usersTab.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "GB Чат 3000"))
        self.UpdateContsBT.setText(_translate("MainWindow", "Обновить"))
        self.usersTab.setTabText(self.usersTab.indexOf(self.Conts), _translate("MainWindow", "Контакты"))
        self.UpdateUsersBT.setText(_translate("MainWindow", "Обновить"))
        self.usersTab.setTabText(self.usersTab.indexOf(self.Users), _translate("MainWindow", "Добавить"))
        self.ContactStatus.setText(_translate("MainWindow", "Онлайн: "))
        self.ContactName.setText(_translate("MainWindow", "Вася"))
        self.SettingsBT.setText(_translate("MainWindow", "Настройки"))
        self.EnterBT.setText(_translate("MainWindow", "Ввод"))
